<?php
// Individual List
//
// NOTE: indilist.php and famlist.php contain mostly identical code.
// Updates to one file almost certainly need to be made to the other one as well.
//
// webtrees: Web based Family History software
// Copyright (C) 2011 webtrees development team.
//
// Derived from PhpGedView
// Copyright (C) 2002 to 2009 PGV Development Team.  All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: indilist.php 12351 2011-10-21 09:47:01Z greg $

define('WT_SCRIPT_NAME', 'indilist.php');
require './includes/session.php';
require_once WT_ROOT.'includes/functions/functions_print_lists.php';

// We show three different lists: initials, surnames and individuals
// Note that the data may contain special chars, such as surname="<unknown>",
$alpha   =safe_GET('alpha', WT_REGEX_UNSAFE); // All surnames beginning with this letter where "@"=unknown and ","=none
$surname =safe_GET('surname', WT_REGEX_UNSAFE); // All indis with this surname.  NB - allow ' and "
$show_all=safe_GET('show_all', array('no','yes'), 'no'); // All indis

// Don't show the list until we have some filter criteria
if (isset($alpha) || isset($surname) || $show_all=='yes') {
	$showList = true;
} else {
	$showList = false;
}

// Long lists can be broken down by given name
$falpha=safe_GET('falpha'); // All first names beginning with this letter
$show_all_firstnames=safe_GET('show_all_firstnames', array('no','yes'), 'no');

// We can show either a list of surnames or a list of names
$surname_sublist=safe_GET('surname_sublist', array('no','yes'));
if (!$surname_sublist) {
	$surname_sublist=safe_COOKIE('surname_sublist', array('no','yes'), 'yes');
}
setcookie('surname_sublist', $surname_sublist);

// We can either include or exclude married names.
// We use $SHOW_MARRIED_NAMES as the default value.
$show_marnm=safe_GET('show_marnm', array('no','yes'));
if (!$show_marnm) {
	$show_marnm=safe_COOKIE('show_marnm_indilist', array('no','yes'));
}
if (!$show_marnm) {
	$show_marnm=$SHOW_MARRIED_NAMES ? 'yes' : 'no';
}
setcookie('show_marnm_indilist', $show_marnm);
// Override $SHOW_MARRIED_NAMES for this page
$SHOW_MARRIED_NAMES=($show_marnm=='yes');

// Fetch a list of the initial letters of all surnames in the database
$initials=WT_Query_Name::surnameAlpha($SHOW_MARRIED_NAMES, false, WT_GED_ID);

// Make sure selections are consistent.
// i.e. can't specify show_all and surname at the same time.
if ($show_all=='yes') {
	$alpha='';
	$surname='';
	$legend=WT_I18N::translate('All');
	$url='indilist.php?show_all=yes';
} elseif ($surname) {
	$alpha=WT_Query_Name::initialLetter($surname);
	$show_all='no';
	if ($surname=='@N.N.') {
		$legend=$UNKNOWN_NN;
	} else {
		$legend=htmlspecialchars($surname);
	}
	switch($falpha) {
	case '':
		break;
	case '@':
		$legend.=', '.$UNKNOWN_PN;
		break;
	default:
		$legend.=', '.htmlspecialchars($falpha);
		break;
	}
	$surname_sublist='no';
	$url='indilist.php?surname='.rawurlencode($surname);
} else {
	$show_all='no';
	$surname='';
	if ($alpha=='@') {
		$legend=$UNKNOWN_NN;
		$surname_sublist='no';
		$surname='@N.N.';
	} elseif ($alpha==',') {
		$legend=WT_I18N::translate('None');
		$surname_sublist='no';
	} else {
		$legend=htmlspecialchars($alpha);
	}
	$url='indilist.php?alpha='.rawurlencode($alpha);
}

print_header(WT_I18N::translate('Individuals').' : '.$legend);
echo '<h2 class="center">', WT_I18N::translate('Individuals'), '</h2>';

// Print a selection list of initial letters
foreach ($initials as $letter=>$count) {
	switch ($letter) {
	case '@':
		$html=$UNKNOWN_NN;
		break;
	case ',':
		$html=WT_I18N::translate('None');
		break;
	default:
		$html=htmlspecialchars($letter);
		break;
	}
	if ($count) {
		if ($showList && $letter==$alpha && $show_all=='no') {
			if ($surname) {
				$html='<a href="indilist.php?alpha='.rawurlencode($letter).'&amp;ged='.WT_GEDURL.'" class="warning" title="'.$count.'">'.$html.'</a>';
			} else {
				$html='<span class="warning" title="'.$count.'">'.$html.'</span>';
			}
		} else {
			$html='<a href="indilist.php?alpha='.rawurlencode($letter).'&amp;ged='.WT_GEDURL.'" title="'.$count.'">'.$html.'</a>';
		}
	}
	$list[]=$html;
}

// Search spiders don't get the "show all" option as the other links give them everything.
if (!$SEARCH_SPIDER) {
	if ($show_all=='yes') {
		$list[]='<span class="warning">'.WT_I18N::translate('All').'</span>';
	} else {
		$list[]='<a href="indilist.php?show_all=yes'.'&amp;ged='.WT_GEDURL.'">'.WT_I18N::translate('All').'</a>';
	}
}
echo '<div class="alpha_index"><p class="center">';
echo WT_I18N::translate('Choose a letter to show individuals whose family name starts with that letter.');
echo help_link('alpha');
echo '<br />', join(' | ', $list), '</p>';

// Search spiders don't get an option to show/hide the surname sublists,
// nor does it make sense on the all/unknown/surname views
if (!$SEARCH_SPIDER) {
	echo '<p class="center">';
	if ($alpha!='@' && $alpha!=',' && !$surname) {
		if ($surname_sublist=='yes') {
			echo '<a href="', $url, '&amp;surname_sublist=no'.'&amp;ged='.WT_GEDURL.'">', WT_I18N::translate('Skip surname lists'), '</a>';
			echo help_link('skip_sublist');
		} else {
			echo '<a href="', $url, '&amp;surname_sublist=yes'.'&amp;ged='.WT_GEDURL.'">', WT_I18N::translate('Show surname lists'), '</a>';
			echo help_link('skip_sublist');
		}
		echo '&nbsp;&nbsp;&nbsp;';
	}
	if ($showList) {
		if ($SHOW_MARRIED_NAMES) {
			echo '<a href="', $url, '&amp;show_marnm=no'.'&amp;ged='.WT_GEDURL.'">', WT_I18N::translate('Exclude married names'), '</a>';
		} else {
			echo '<a href="', $url, '&amp;show_marnm=yes'.'&amp;ged='.WT_GEDURL.'">', WT_I18N::translate('Include married names'), '</a>';
		}
		echo help_link('show_marnm');
	}
	echo '</p>';
}
echo '</div>';

if ($showList) {
	$surns=WT_Query_Name::surnames($surname, $alpha, $SHOW_MARRIED_NAMES, false, WT_GED_ID);
	if ($surname_sublist=='yes') {
		// Show the surname list
		switch ($SURNAME_LIST_STYLE) {
		case 'style1';
			echo format_surname_list($surns, 3, true, 'indilist');
			break;
		case 'style3':
			echo format_surname_tagcloud($surns, 'indilist', true);
			break;
		case 'style2':
		default:
			echo format_surname_table($surns, 'indilist');
			break;
		}
	} else {
		// Show the individual list
		$count=0;
		foreach ($surns as $surnames) {
			foreach ($surnames as $list) {
				$count+=count($list);
			}
		}
		// Don't sublists short lists.
		if ($count<get_gedcom_setting(WT_GED_ID, 'SUBLIST_TRIGGER_I')) {
			$falpha='';
			$show_all_firstnames='no';
		} else {
			$givn_initials=WT_Query_Name::givenAlpha($surname, $alpha, $SHOW_MARRIED_NAMES, false, WT_GED_ID);
			// Break long lists by initial letter of given name
			if ($surname || $show_all=='yes') {
				// Don't show the list until we have some filter criteria
				$showList=($falpha || $show_all_firstnames=='yes');
				$list=array();
				foreach ($givn_initials as $givn_initial=>$count) {
					switch ($givn_initial) {
					case '@':
						$html=$UNKNOWN_PN;
						break;
					default:
						$html=htmlspecialchars($givn_initial);
						break;
					}
					if ($count) {
						if ($showList && $givn_initial==$falpha && $show_all_firstnames=='no') {
							$html='<span class="warning" title="'.$count.'">'.$html.'</span>';
						} else {
							$html='<a href="'.$url.'&amp;falpha='.rawurlencode($givn_initial).'&amp;ged='.WT_GEDURL.'" title="'.$count.'">'.$html.'</a>';
						}
					}
					$list[]=$html;
				}
				// Search spiders don't get the "show all" option as the other links give them everything.
				if (!$SEARCH_SPIDER) {
					if ($show_all_firstnames=='yes') {
						$list[]='<span class="warning">'.WT_I18N::translate('All').'</span>';
					} else {
						$list[]='<a href="'.$url.'&amp;show_all_firstnames=yes'.'&amp;ged='.WT_GEDURL.'">'.WT_I18N::translate('All').'</a>';
					}
				}
				if ($show_all=='no') {
					echo '<h2 class="center">';
					echo WT_I18N::translate('Individuals with surname %s', check_NN($surname));
					echo '</h2>';
				}
				echo '<div class="alpha_index"><p class="center">';
				echo WT_I18N::translate('Choose a letter to show individuals who have a given name which starts with that letter.');
				echo help_link('alpha');
				echo '<br />', join(' | ', $list), '</p></div>';
			}
		}
		if ($showList) {
			if ($legend && $show_all=='no') {
				$legend=WT_I18N::translate('Individuals with surname %s', check_NN($legend));
			}
			$indis=WT_Query_Name::individuals($surname, $alpha, $falpha, $SHOW_MARRIED_NAMES, false, WT_GED_ID);
			print_indi_table($indis, $legend);
		}
	}
}

print_footer();
